@@ -36,12 +36,25 @@ from azureml.exceptions import ServiceException
 
 _LOCK = Lock()
 _PREFIX = None
+_SCRUB_MESSAGE = "**Log message scrubbed**"
+_SCRUB_LOGGING = False
 _AML_RUN = None
 
 
+def set_scrubbed_logging(value: bool) -> None:
+    """
+    Set the global scrubbed logging functionality settings.
+
+    This method is thread-safe.
+    """
+    with _LOCK:
+        global _SCRUB_LOGGING
+        _SCRUB_LOGGING = value
+
+
 def set_prefix(prefix: str) -> None:
     """
-    Set the global prefix to use when logging public (non-private) data.
+    Set the global prefix to use when logging data.
 
     This method is thread-safe.
     """
@@ -50,14 +63,40 @@ def set_prefix(prefix: str) -> None:
         _PREFIX = prefix
 
 
+def set_scrub_message(scrub_message: str) -> None:
+    """
+    Set the global scrub message to use when logging private (non-public) data.
+
+    This method is thread-safe.
+    """
+    with _LOCK:
+        global _SCRUB_MESSAGE
+        # Make the passed in value string as dditional safety measure
+        _SCRUB_MESSAGE = str(scrub_message)
+
+
+def get_scrubbed_logging() -> bool:
+    """
+    Obtain the current global scrubbed logging functionality settings.
+    """
+    return _SCRUB_LOGGING
+
+
 def get_prefix() -> Optional[str]:
     """
-    Obtain the current global prefix to use when logging public (non-private)
-    data.
+    Obtain the current global prefix to use when logging data.
     """
     return _PREFIX
 
 
+def get_scrub_message() -> str:
+    """
+    Obtain the current global scrub message to use when logging private (non-public)
+    data.
+    """
+    return _SCRUB_MESSAGE
+
+
 def set_aml_context() -> None:
     """
     Retrieves the AML Context, should be bundled in a try-catch.
@@ -65,7 +104,7 @@ def set_aml_context() -> None:
     global _AML_RUN
     from azureml.core.run import Run
 
-    _AML_RUN = Run.get_context()
+    _AML_RUN = Run.get_context(allow_offline=False)
 
 
 def get_aml_context():
@@ -207,9 +246,18 @@ class CompliantLogger(logging.getLoggerClass()):  # type: ignore
         items=None,
         category=DataCategory.PRIVATE,
     ):
-        p = ""
-        if category == DataCategory.PUBLIC:
+
+        if not get_scrubbed_logging() and category == DataCategory.PRIVATE:
+            p = ""
+        else:
             p = get_prefix()
+
+        if get_scrubbed_logging() and category == DataCategory.PRIVATE:
+            msg = get_scrub_message()
+            items = None
+            stack_info = False
+            exc_info = None
+
         if extra:
             extra.update({"prefix": p})
         else:
@@ -288,7 +336,8 @@ class CompliantLogger(logging.getLoggerClass()):  # type: ignore
         # check if value provided
         if value is None:
             self.error(
-                f"Value provided for metric {name} is None, skipping (step: {step})"
+                f"Value provided for metric {name} is None, skipping (step: {step})",
+                category=category,
             )
             return
 
@@ -320,7 +369,10 @@ class CompliantLogger(logging.getLoggerClass()):  # type: ignore
         if isinstance(value, dict):
             # check if values are present
             if len(value) == 0:
-                self.warning(f"Dictionary Value for Metric {name} is empty. Skipping.")
+                self.warning(
+                    f"Dictionary Value for Metric {name} is empty. Skipping.",
+                    category=category,
+                )
                 return
 
             # check the value types of the dict
@@ -382,14 +434,17 @@ class CompliantLogger(logging.getLoggerClass()):  # type: ignore
 
             # check if values are present
             if len(value) == 0:
-                self.warning(f"List Value for Metric {name} is empty. Skipping.")
+                self.warning(
+                    f"List Value for Metric {name} is empty. Skipping.",
+                    category=category,
+                )
                 return
 
             # log data to run context
             if run is not None and category == DataCategory.PUBLIC:
                 run.log_list(name=name, value=value, description=description)
             else:
-                self.info(f"ListMetric      | {name} | {value}")
+                self.info(f"ListMetric      | {name} | {value}", category=category)
 
             return
 
@@ -445,7 +500,16 @@ class CompliantLogger(logging.getLoggerClass()):  # type: ignore
 
         # check if parameters are correct
         if category != DataCategory.PUBLIC:
-            self.warning(f"Unable to log image metric {name} as private, skipping.")
+            self.warning(
+                f"Unable to log image metric {name} as private, skipping.",
+                category=DataCategory.PUBLIC,
+            )
+            return
+        elif run is None:
+            self.warning(
+                f"Unable to log image metric {name} without AML Run Context, skipping.",
+                category=category,
+            )
             return
 
         # check for name
@@ -463,7 +527,7 @@ class CompliantLogger(logging.getLoggerClass()):  # type: ignore
         except ServiceException:
             self.warning(
                 "log_image is not available for detonation chamber, skipping.",
-                category=DataCategory.PUBLIC,
+                category=category,
             )
 
     def metric_list(self, name, value, description=None, category=DataCategory.PRIVATE):
@@ -634,7 +698,10 @@ class CompliantLogger(logging.getLoggerClass()):  # type: ignore
         if category == DataCategory.PUBLIC and run is not None:
             run.log_accuracy_table(name, value, description)
         else:
-            self.warning("Logging Accuracy Tables to text is not yet implemented")
+            self.warning(
+                "Logging Accuracy Tables to text is not yet implemented",
+                category=DataCategory.PUBLIC,
+            )
 
     def metric_confusion_matrix(
         self,
@@ -733,7 +800,10 @@ class CompliantLogger(logging.getLoggerClass()):  # type: ignore
         if category == DataCategory.PUBLIC and run is not None:
             run.log_confusion_matrix(name, value, description)
         else:
-            self.warning("Logging Confusion Matrices to text is not yet implemented")
+            self.warning(
+                "Logging Confusion Matrices to text is not yet implemented",
+                category=DataCategory.PUBLIC,
+            )
 
     def metric_predictions(
         self,
@@ -833,7 +903,10 @@ class CompliantLogger(logging.getLoggerClass()):  # type: ignore
         if category == DataCategory.PUBLIC and run is not None:
             run.log_predictions(name, value, description)
         else:
-            self.warning("Logging Predictions to text is not yet implemented")
+            self.warning(
+                "Logging Predictions to text is not yet implemented",
+                category=DataCategory.PUBLIC,
+            )
 
     def metric_residual(
         self,
@@ -930,7 +1003,10 @@ class CompliantLogger(logging.getLoggerClass()):  # type: ignore
         if category == DataCategory.PUBLIC and run is not None:
             run.log_residuals(name, value, description)
         else:
-            self.warning("Logging Residuals to text is not yet implemented")
+            self.warning(
+                "Logging Residuals to text is not yet implemented",
+                category=DataCategory.PUBLIC,
+            )
 
     def metric_row(
         self, name, description=None, category=DataCategory.PRIVATE, **kwargs
@@ -998,7 +1074,11 @@ https://github.com/Azure/confidential-ml-utils/issues/33 for more information.
 
 
 def enable_compliant_logging(
-    prefix: str = "SystemLog:", use_aml_metrics: bool = False, **kwargs
+    prefix: str = "SystemLog:",
+    scrub_message: str = "**Log message scrubbed**",
+    use_aml_metrics: bool = False,
+    enable_scrubbed_logging: bool = False,
+    **kwargs,
 ) -> None:
     """
     The default format is `logging.BASIC_FORMAT` (`%(levelname)s:%(name)s:%(message)s`).
@@ -1020,6 +1100,10 @@ def enable_compliant_logging(
     https://github.com/python/cpython/blob/3.9/Lib/logging/__init__.py
     """
     set_prefix(prefix)
+    set_scrub_message(scrub_message)
+
+    if enable_scrubbed_logging:
+        set_scrubbed_logging(True)
 
     if "format" not in kwargs:
         kwargs["format"] = f"%(prefix)s{logging.BASIC_FORMAT}"
@@ -1050,7 +1134,11 @@ def enable_compliant_logging(
 
 
 def enable_confidential_logging(
-    prefix: str = "SystemLog:", use_aml_metrics: bool = False, **kwargs
+    prefix: str = "SystemLog:",
+    scrub_message: str = "**Log message scrubbed**",
+    use_aml_metrics: bool = False,
+    enable_scrubbed_logging: bool = False,
+    **kwargs,
 ) -> None:
     """
     This function is a duplicate of the function `enable_compliant_logging`.
@@ -1061,4 +1149,6 @@ def enable_confidential_logging(
         " to deprecation. Please use enable_compliant_logging() instead.",
         file=sys.stderr,
     )
-    enable_compliant_logging(prefix, use_aml_metrics, **kwargs)
+    enable_compliant_logging(
+        prefix, scrub_message, use_aml_metrics, enable_scrubbed_logging, **kwargs
+    )
