@@ -106,13 +106,19 @@ class pipeline_compute_config:  # pylint: disable=invalid-name
     - `windows_output_mode` (str): output mode for windows compute targets (default to "upload"), more details at https://componentsdk.azurewebsites.net/concepts/inputs-and-outputs.html#output-dataset-mode
     - `hdi_prod_target` (str): name of HDI regular compute target
     - `hdi_dc_target` (Optional[str]): name of HDI detonation chamber compute target
-    - `synapse_prod_target` (str): name of Synapse regular compute target
     - `hdi_driver_memory` (str): HDI driver memory in GB (default to "4g")
     - `hdi_driver_cores` (int): number of HDI driver cores (default to 2)
     - `hdi_executor_memory` (str): HDI executor memory in GB (default to "3g")
     - `hdi_executor_cores` (int): number of HDI executor cores (default to 2)
     - `hdi_number_executors` (int): number of HDI executors (default to 1)
     - `hdi_conf` (Optional[Any]): additional HDI parameters
+    - `synapse_prod_target` (str): name of Synapse regular compute target
+    - `synapse_driver_memory` (Optional[str]): Synapse driver memory in GB
+    - `synapse_driver_cores` (Optional[int]): number of Synapse driver cores
+    - `synapse_executor_memory` (Optional[str]): Synapse executor memory in GB
+    - `synapse_executor_cores` (Optional[int]): number of Synapse executor cores
+    - `synapse_number_executors` (Optional[int]): number of Synapse executors
+    - `synapse_conf` (Optional[Any]): additional Synapse parameters
     - `parallel_node_count` (int): number of nodes for parallel steps (default to 10)
     - `parallel_process_count_per_node` (Optional[int]): number of processes for each node for parallel steps
     - `parallel_run_invocation_timeout` (int): threshold in seconds for parallel step invocations to timeout (default to 10800)
@@ -140,7 +146,6 @@ class pipeline_compute_config:  # pylint: disable=invalid-name
 
     hdi_prod_target: str = MISSING
     hdi_dc_target: Optional[str] = None
-    synapse_prod_target: Optional[str] = None
     hdi_driver_memory: str = "4g"
     hdi_driver_cores: int = 2
     hdi_executor_memory: str = "3g"
@@ -148,6 +153,14 @@ class pipeline_compute_config:  # pylint: disable=invalid-name
     hdi_number_executors: int = 10
     hdi_conf: Optional[Any] = MISSING
 
+    synapse_prod_target: Optional[str] = None
+    synapse_driver_memory: Optional[str] = MISSING
+    synapse_driver_cores: Optional[int] = MISSING
+    synapse_executor_memory: Optional[str] = MISSING
+    synapse_executor_cores: Optional[int] = MISSING
+    synapse_number_executors: Optional[int] = MISSING
+    synapse_conf: Optional[Any] = MISSING
+
     parallel_node_count: int = 10
     parallel_process_count_per_node: Optional[int] = MISSING
     parallel_run_invocation_timeout: int = 10800
