@@ -124,6 +124,14 @@ class AMLPipelineHelper:
         """
         raise NotImplementedError("You need to implement your build() method.")
 
+    def extra_tags(self):
+        """Additional tags to submit with the pipeline
+
+        Returns:
+            dict[str, str]: tags
+        """
+        return {}
+
     def pipeline_instance(self, pipeline_function, config):
         """Creates an instance of the pipeline using arguments.
 
@@ -915,6 +923,7 @@ class AMLPipelineHelper:
         scope_param=None,
         custom_job_name_suffix=None,
         adla_account_name=None,
+        priority=None,
         **custom_runtime_arguments,
     ):
         """Applies settings for a scope module.
@@ -927,6 +936,7 @@ class AMLPipelineHelper:
             scope_param (str): Parameters to pass to scope e.g. Nebula parameters, VC allocation parameters etc.
             custom_job_name_suffix (str): Optional parameter defining custom string to append to job name
             adla_account_name (str): The name of the Cosmos-migrated Azure Data Lake Analytics account to submit scope job
+            priority (int): Scope job priority. Default is 1000.
             custom_runtime_arguments (dict): any additional custom args
         """
         log.info(
@@ -943,6 +953,7 @@ class AMLPipelineHelper:
             adla_account_name=adla_account_name,
             scope_param=scope_param,
             custom_job_name_suffix=custom_job_name_suffix,
+            priority=priority,
         )
 
     def _apply_datatransfer_runsettings(
@@ -1175,7 +1186,7 @@ class AMLPipelineHelper:
         `component_name = component_instance.name`. Otherwise
         `apply_smart_runsettings()` might work incorrectly for local
         components. See more detailed explanation and examples here:
-        https://github.com/Azure/shrike/issues/411
+        https://github.com/ai-platform-ml-platform/shrike/issues/411
         """
         component_manifest_list = self.config.modules.manifest
         component_name = component_instance.name
@@ -1309,7 +1320,7 @@ class AMLPipelineHelper:
                 log.info(f"Module {module_name} detected as SweepComponent: {sweep}")
 
         if synapse == "auto":
-            synapse = str(module_instance.type) == "spark"
+            synapse = str(module_instance.type) == "SparkComponent"
             if synapse:
                 log.info(f"Module {module_name} detected as spark (Synapse): {synapse}")
 
@@ -1553,7 +1564,7 @@ class AMLPipelineHelper:
                             )
                     else:
                         log.info(
-                            f"Override for key {key} is not supported yet. Please open a [feature request](https://github.com/Azure/shrike/issues) if necessary."
+                            f"Override for key {key} is not supported yet. Please open a [feature request](https://github.com/ai-platform-ml-platform/shrike/issues) if necessary."
                         )
                 except KeyError:
                     log.info(f"Key {key} does not in file {spec_path}. Skip overrides.")
@@ -1563,9 +1574,12 @@ class AMLPipelineHelper:
             spec_dirname = os.path.dirname(spec_path)
             log.info(f"Will remove {POLYMER_FEED} from environment.conda.")
             try:
-                conda_dependencies_file = spec["environment"]["conda"][
-                    "conda_dependencies_file"
-                ]
+                if spec["type"] == "spark":
+                    conda_dependencies_file = spec["environment"]["conda_file"]
+                else:
+                    conda_dependencies_file = spec["environment"]["conda"][
+                        "conda_dependencies_file"
+                    ]
                 log.info("conda_dependencies_file exists.")
                 (
                     found_index_url,
@@ -1576,14 +1590,22 @@ class AMLPipelineHelper:
                     spec_dirname, conda_dependencies_file, POLYMER_FEED
                 )
                 if found_index_url:
-                    spec["environment"]["conda"]["conda_dependencies_file"] = new_file
+                    if spec["type"] == "spark":
+                        spec["environment"]["conda_file"] = new_file
+                    else:
+                        spec["environment"]["conda"][
+                            "conda_dependencies_file"
+                        ] = new_file
             except KeyError:
                 # conda_dependencies_file does not exist
                 pass
             try:
-                conda_dependencies = spec["environment"]["conda"]["conda_dependencies"][
-                    "dependencies"
-                ]
+                if spec["type"] == "spark":
+                    conda_dependencies = spec["conda_dependencies"]["dependencies"]
+                else:
+                    conda_dependencies = spec["environment"]["conda"][
+                        "conda_dependencies"
+                    ]["dependencies"]
                 log.info("conda_dependencies_file exists.")
                 for idx, dependency in enumerate(conda_dependencies):
                     if isinstance(dependency, dict) and "pip" in dependency:
@@ -1592,9 +1614,12 @@ class AMLPipelineHelper:
                             pip_dependencies.remove(POLYMER_FEED)
                             dependency["pip"] = pip_dependencies
                             conda_dependencies[idx] = dependency
-                spec["environment"]["conda"]["conda_dependencies"][
-                    "dependencies"
-                ] = conda_dependencies
+                if spec["type"] == "spark":
+                    spec["conda_dependencies"]["dependencies"] = conda_dependencies
+                else:
+                    spec["environment"]["conda"]["conda_dependencies"][
+                        "dependencies"
+                    ] = conda_dependencies
             except KeyError:
                 # conda_dependencies does not exist
                 pass
@@ -1699,11 +1724,21 @@ class AMLPipelineHelper:
             node_mapping[n._instance_id] = n
             for input in n.inputs.values():
                 try:
-                    owner = input._get_data_owner()
+                    owner = (
+                        input._get_data_owner()
+                    )  # points to subgraph node if upstream component is subgraph
                     if owner:
                         graph[n._instance_id].append(owner._instance_id)
+
                 except AttributeError:
                     continue
+        subpipelines = pipeline._expand_pipeline_to_pipelines()
+        for n in subpipelines:
+            # https://msdata.visualstudio.com/DefaultCollection/Vienna/_workitems/edit/2026740/
+            # if the last node of a subgraph needs to be updated
+            # then we need to update subgraph.outputs datastore too
+            node_mapping[n._instance_id] = n
+            graph[n._instance_id].append(n.nodes[-1]._instance_id)
         sorted_node_ids = toposort_flatten(graph)
         use_hdi_dc = defaultdict(bool)
         for id in sorted_node_ids:
@@ -1721,8 +1756,8 @@ class AMLPipelineHelper:
                     use_hdi_dc[id] = True
                     break
 
-        for node in nodes:
-            if use_hdi_dc[node._instance_id]:
+        for id, node in node_mapping.items():
+            if use_hdi_dc[id]:
                 if node.type == "ScopeComponent":
                     # scope component doesn't need to configure compute target
                     # output datastore has to be cosmos (ADLSg1), while dc_datastore is ADLSg2
@@ -1759,7 +1794,7 @@ class AMLPipelineHelper:
                 else:
                     continue
                 log.info(
-                    f"Updating compute target for {node.name} {node._instance_id} to {node.runsettings.target}"
+                    f"Updating compute target for {node.name} {id} to {node.runsettings.target}"
                 )
         return pipeline
 
@@ -1782,6 +1817,19 @@ class AMLPipelineHelper:
             f"Publishing pipeline as endpoint {endpoint_name} with name {name} to workspace {self.workspace()}."
         )
 
+    def _validate_tags(self, tags: dict):
+        vaidated_tags = {}
+        if isinstance(tags, dict):
+            # NOTE: values for tags need to be str
+            for key, value in tags.items():
+                if not isinstance(key, str):
+                    log.warning(f"The key for pipeline tag {key} is not string")
+                else:
+                    vaidated_tags[key] = str(value)
+        else:
+            log.warning(f"The pipeline tags {tags} value type is not a dict")
+        return vaidated_tags
+
     ################
     ### MAIN/RUN ###
     ################
@@ -1827,6 +1875,7 @@ class AMLPipelineHelper:
             pipeline_tags = self._parse_pipeline_tags()
             pipeline_tags.update({"shrike": __version__})
             pipeline_tags.update(self.repository_info)
+            pipeline_tags.update(self._validate_tags(self.extra_tags()))
             log.info(f"Submitting Experiment... [tags={pipeline_tags}]")
 
             # pipeline_run is of the class "azure.ml.component.run", which
